@@ -32,6 +32,7 @@ class BucketizeBatchSampler(BatchSampler):
             (Default: ``None``)
         shuffle (bool, optional): Whether to shuffle buckets for non-monotonic length sampling.
             (Default: True)
+        seed (int, optional): The seed for initialzing RNG. Only used when `shuffle` is True. (Default: 0)
         drop_last (bool, optional): If ``True``, the sampler will drop the last batch if
             its size would be less than ``batch_size``
             (Default: False)
@@ -45,7 +46,7 @@ class BucketizeBatchSampler(BatchSampler):
 
     Note:
         if ``shuffle`` is True, it will only shuffle the data once. Please set ``reload_dataloaders_every_n_epochs=1``
-        in pytorch_lightning Trainer to enable shuffling every epoch.
+        in pytorch_lightning Trainer and set ``seed`` to ``self.trainer.current_epoch`` to enable shuffling every epoch.
     """
 
     def __init__(
@@ -57,6 +58,7 @@ class BucketizeBatchSampler(BatchSampler):
         max_token_count: Optional[int] = None,
         batch_size: Optional[int] = None,
         shuffle: bool = True,
+        seed: int = 0,
         drop_last: bool = False,
     ) -> None:
         if max_len is None:
@@ -82,6 +84,10 @@ class BucketizeBatchSampler(BatchSampler):
         self.max_token_count = max_token_count
         self.batch_size = batch_size
         self.shuffle = shuffle
+        self.seed = seed
+        if self.shuffle:
+            self.g = torch.Generator()
+            self.g.manual_seed(self.seed)
         self.drop_last = drop_last
         self.buckets = self._get_buckets(self.lengths, num_buckets, min_len, max_len)
         self._update_iter_list()
@@ -115,7 +121,7 @@ class BucketizeBatchSampler(BatchSampler):
     def _update_iter_list(self) -> None:
         if self.shuffle:
             for k in self.buckets:
-                self.buckets[k] = self.buckets[k][torch.randperm(self.buckets[k].size(0))]
+                self.buckets[k] = self.buckets[k][torch.randperm(self.buckets[k].size(0), generator=self.g)]
         self.iter_list = []
         total_len = 0
         batch = []
@@ -193,7 +199,18 @@ class DistributedBatchSampler(DistributedSampler):
         self.epoch = 0
         self.seed = seed
         self.drop_last = drop_last
-        if shuffle:
+        self.shuffle = shuffle
+        indices = self.batch_sampler.iter_list
+        if self.drop_last and len(indices) % self.num_replicas != 0:
+            # Split to nearest available length that is evenly divisible.
+            # This is to ensure each rank receives the same amount of data when
+            # using this Sampler.
+            self.num_samples = math.ceil((len(indices) - self.num_replicas) / self.num_replicas)
+        else:
+            self.num_samples = math.ceil(len(indices) / self.num_replicas)
+
+    def __iter__(self):
+        if self.shuffle:
             g = torch.Generator()
             g.manual_seed(self.seed + self.epoch)
             perm = torch.randperm(len(self.batch_sampler.iter_list), generator=g).tolist()
@@ -210,7 +227,6 @@ class DistributedBatchSampler(DistributedSampler):
         self.subset = indices[self.rank : self.total_size : self.num_replicas]
         assert len(self.subset) == self.num_samples
 
-    def __iter__(self):
         return iter(self.subset)
 
     def __len__(self):
