@@ -1,21 +1,29 @@
-from .backend_utils import set_audio_backend
+from .autograd_utils import use_deterministic_algorithms
 from .case_utils import (
+    disabledInCI,
     HttpServerMixin,
-    is_ffmpeg_available,
     PytorchTestCase,
+    skipIfCudaSmallMemory,
+    skipIfNoAudioDevice,
     skipIfNoCtcDecoder,
+    skipIfNoCuCtcDecoder,
     skipIfNoCuda,
     skipIfNoExec,
     skipIfNoFFmpeg,
-    skipIfNoKaldi,
+    skipIfNoHWAccel,
+    skipIfNoMacOS,
     skipIfNoModule,
     skipIfNoQengine,
+    skipIfNoRIR,
     skipIfNoSox,
+    skipIfNoSoxDecoder,
+    skipIfNoSoxEncoder,
     skipIfPy310,
     skipIfRocm,
     TempDirMixin,
     TestBaseMixin,
     TorchaudioTestCase,
+    zip_equal,
 )
 from .data_utils import get_asset_path, get_sinusoid, get_spectrogram, get_whitenoise
 from .func_utils import torch_script
@@ -28,24 +36,29 @@ __all__ = [
     "get_whitenoise",
     "get_sinusoid",
     "get_spectrogram",
-    "set_audio_backend",
     "TempDirMixin",
     "HttpServerMixin",
     "TestBaseMixin",
     "PytorchTestCase",
     "TorchaudioTestCase",
-    "is_ffmpeg_available",
+    "skipIfNoAudioDevice",
     "skipIfNoCtcDecoder",
+    "skipIfNoCuCtcDecoder",
     "skipIfNoCuda",
+    "skipIfCudaSmallMemory",
     "skipIfNoExec",
+    "skipIfNoMacOS",
     "skipIfNoModule",
-    "skipIfNoKaldi",
+    "skipIfNoRIR",
     "skipIfNoSox",
-    "skipIfNoSoxBackend",
+    "skipIfNoSoxDecoder",
+    "skipIfNoSoxEncoder",
     "skipIfRocm",
     "skipIfNoQengine",
     "skipIfNoFFmpeg",
+    "skipIfNoHWAccel",
     "skipIfPy310",
+    "disabledInCI",
     "get_wav_data",
     "normalize_wav",
     "load_wav",
@@ -57,4 +70,6 @@ __all__ = [
     "get_image",
     "rgb_to_gray",
     "rgb_to_yuv_ccir",
+    "use_deterministic_algorithms",
+    "zip_equal",
 ]
