@@ -3,7 +3,7 @@ import unittest
 import torch
 from torchaudio_unittest.common_utils import PytorchTestCase, skipIfNoCuda
 
-from .functional_impl import Functional
+from .functional_impl import Functional, FunctionalCUDAOnly
 
 
 @skipIfNoCuda
@@ -20,3 +20,15 @@ class TestFunctionalFloat32(Functional, PytorchTestCase):
 class TestLFilterFloat64(Functional, PytorchTestCase):
     dtype = torch.float64
     device = torch.device("cuda")
+
+
+@skipIfNoCuda
+class TestFunctionalCUDAOnlyFloat32(FunctionalCUDAOnly, PytorchTestCase):
+    dtype = torch.float32
+    device = torch.device("cuda")
+
+
+@skipIfNoCuda
+class TestFunctionalCUDAOnlyFloat64(FunctionalCUDAOnly, PytorchTestCase):
+    dtype = torch.float64
+    device = torch.device("cuda")
