@@ -54,24 +54,3 @@ class SmokeTest(TempDirMixin, TorchaudioTestCase):
         _found, _sr = sox_effects.apply_effects_file(
             input_path, effects, normalize=False, channels_first=channels_first
         )
-
-    @parameterized.expand(
-        load_params("sox_effect_test_args.jsonl"),
-        name_func=lambda f, i, p: f'{f.__name__}_{i}_{p.args[0]["effects"][0][0]}',
-    )
-    def test_apply_effects_fileobj(self, args):
-        """`apply_effects_file` should return identical data as sox command"""
-        dtype = "int32"
-        channels_first = True
-        effects = args["effects"]
-        num_channels = args.get("num_channels", 2)
-        input_sr = args.get("input_sample_rate", 8000)
-
-        input_path = self.get_temp_path("input.wav")
-        data = get_wav_data(dtype, num_channels, channels_first=channels_first)
-        save_wav(input_path, data, input_sr, channels_first=channels_first)
-
-        with open(input_path, "rb") as fileobj:
-            _found, _sr = sox_effects.apply_effects_file(
-                fileobj, effects, normalize=False, channels_first=channels_first
-            )
